package FDN::Sql::MultiDBC;

use strict;
use warnings;
use base 'Exporter';

use FDN::Common::Logger;
use FDN::Sql::Basic;
use FDN::Sql::DB_Access;
use FDN::Sql::DBC qw();
use FDN::Sql::Structure qw(read_config read_files structure create switch_option set_quiet set_output);


our @EXPORT = qw(
	do_connect
	reset_multi_dbh
	switch_multi_dbh
);

my $dbh;
my $DBS;
my $sub_dbh;

sub __tous_db() {
	return __tous($dbh, 'DB');
}

sub __incr_db_cur($) {
	my ( $dbid ) = @_;
	return __set_tbl_val($dbh, 'DB', 'DB_CUR', dochar($dbid), 'DB_CUR+1');
}

sub __new_base($$) {
	my ( $suffix, $dbname ) = @_;
	return __new($dbh, 'BASE',
		'BASE_SUFFIX='.dochar($suffix),
		'BASE_DBNAME='.dochar($dbname)
	);
}

sub __base_par_suffix($) {
	my ( $suffix ) = @_;
	return __par_clef($dbh, 'BASE', $suffix, 'BASE_SUFFIX');
}

sub __set_base_version($$) {
	my ( $bid, $version ) = @_;
	return __set_tbl_val($dbh, 'BASE', 'BASE_VERSION', dochar($bid), dochar($version));
}

sub __dispatch($) {
	my ( $suffix ) = @_;
	my $dbs = [ __tous_db() ];
	my $limits = [];
	my $total = 0;
	for my $db ( @$dbs ) {
		$total += $db->{'DB_MAX'};
		$total -= $db->{'DB_CUR'};
		push @$limits, $total;
	}

	my $choice = rand($total);
	while ( $limits->[0] < $choice && $limits->[1] ) {
		shift @$limits;
		shift @$dbs;
	}
	my $dbname = $dbs->[0]->{'DB_NAME'};
	my $bid = __new_base($suffix, $dbname);
	if ( ! $bid ) {
		error "Failed to create a line in BASE";
		return undef;
	}
	__incr_db_cur($dbs->[0]->{'DB_ID'});
	return __base_par_suffix($suffix);
}

sub __switch_db($$) {
	my ( $dbh, $dbname ) = @_;
	$DBS->{'name'} = $dbname;
	create($DBS);
	my $res = sql_transaction($dbh, 'use', 'use '.$dbname);
	if ( !$res->{'ok'} ) {
		error "Failed to switch to database '$dbname' : ".$res->{'err_mess'};
		return undef;
	}
	$dbh->{'dbname'} = $dbname;
}

BEGIN {
	switch_option('dry',0,'all');
	set_quiet('quiet',0);
	set_output('syslog');
}

sub do_connect($) {
	my ( $name ) = @_;
	$dbh = FDN::Sql::DBC::do_connect($name);
	$DBS = read_config("check/$name.cfg","multi");
	read_files($DBS);
	return $dbh;
}

sub reset_multi_dbh {
	return switch_multi_dbh('NONE');
}


sub switch_multi_dbh($) {
	my ( $suffix ) = @_;
	if ( ! defined $dbh ) {
		die "Il faut se connecter, quand meme, un peu.";
	}
	if ( ! defined $sub_dbh ) {
		$sub_dbh = clone_dbh($dbh);
		$sub_dbh->{'suffix'} //= 'NONE';
		$sub_dbh->{'dbname'} //= 'NONE';
	}
	if ( $sub_dbh->{'suffix'} eq $suffix ) {
		return $sub_dbh;
	}
	if ( $suffix eq 'NONE' ) {
		$sub_dbh->{'suffix'} = 'NONE';
		return undef;
	}
	my $base = __base_par_suffix($suffix);
	if ( ! defined $base ) {
		$base = __dispatch($suffix);
	}
	if ( $base->{'BASE_DBNAME'} ne $sub_dbh->{'dbname'} ) {
		__switch_db($sub_dbh, $base->{'BASE_DBNAME'});
	}
	$sub_dbh->{'suffix'} = $suffix;
	if ( $base->{'BASE_VERSION'} ne $DBS->{'signature'} ) {
		structure($DBS, clone_dbh($sub_dbh));
		__set_base_version($base->{'BASE_ID'}, $DBS->{'signature'});
	}
	return $sub_dbh;
}




1;
